@@ -48,8 +48,12 @@ const int MP2Decoder::sblimits[] = {
 };
 
 
-MP2Decoder::MP2Decoder(SubchannelSinkObserver* observer) : SubchannelSink(observer, "mp2") {
+MP2Decoder::MP2Decoder(SubchannelSinkObserver* observer, bool float32) : SubchannelSink(observer, "mp2") {
+	this->float32 = float32;
+
 	scf_crc_len = -1;
+	lsf = false;
+
 
 	int mpg_result;
 
@@ -76,11 +80,11 @@ MP2Decoder::MP2Decoder(SubchannelSinkObserver* observer) : SubchannelSink(observ
 	if(mpg_result != MPG123_OK)
 		throw std::runtime_error("MP2Decoder: error while mpg123_format_none: " + std::string(mpg123_plain_strerror(mpg_result)));
 
-	mpg_result = mpg123_format(handle, 48000, MPG123_MONO | MPG123_STEREO, MPG123_ENC_FLOAT_32);
+	mpg_result = mpg123_format(handle, 48000, MPG123_MONO | MPG123_STEREO, float32 ? MPG123_ENC_FLOAT_32 : MPG123_ENC_SIGNED_16);
 	if(mpg_result != MPG123_OK)
 		throw std::runtime_error("MP2Decoder: error while mpg123_format #1: " + std::string(mpg123_plain_strerror(mpg_result)));
 
-	mpg_result = mpg123_format(handle, 24000, MPG123_MONO | MPG123_STEREO, MPG123_ENC_FLOAT_32);
+	mpg_result = mpg123_format(handle, 24000, MPG123_MONO | MPG123_STEREO, float32 ? MPG123_ENC_FLOAT_32 : MPG123_ENC_SIGNED_16);
 	if(mpg_result != MPG123_OK)
 		throw std::runtime_error("MP2Decoder: error while mpg123_format #2: " + std::string(mpg123_plain_strerror(mpg_result)));
 
@@ -151,7 +155,7 @@ size_t MP2Decoder::DecodeFrame(uint8_t **data) {
 
 	// check CRC (MP2's CRC only - not DAB's ScF-CRC)
 	if(!CheckCRC(header, body_data, body_bytes)) {
-		fprintf(stderr, "\x1B[31m" "(CRC)" "\x1B[0m" " ");
+		observer->AudioError("CRC");
 		// no PAD reset, as not covered by CRC
 		return 0;
 	}
@@ -183,7 +187,7 @@ void MP2Decoder::ProcessUntouchedStream(const unsigned long& header, const uint8
 	frame[3] = header & 0xFF;
 	memcpy(&frame[4], body_data, body_bytes);
 
-	ForwardUntouchedStream(&frame[0], frame.size());
+	ForwardUntouchedStream(&frame[0], frame.size(), lsf ? 48 : 24);
 }
 
 bool MP2Decoder::CheckCRC(const unsigned long& header, const uint8_t *body_data, const size_t& body_bytes) {
@@ -254,7 +258,7 @@ void MP2Decoder::ProcessFormat() {
 	scf_crc_len = (info.version == MPG123_1_0 && info.bitrate < (info.mode == MPG123_M_MONO ? 56 : 112)) ? 2 : 4;
 
 	// output format
-	const char* version = "unknown";
+	std::string version = "unknown";
 	switch(info.version) {
 	case MPG123_1_0:
 		version = "1.0";
@@ -266,8 +270,9 @@ void MP2Decoder::ProcessFormat() {
 		version = "2.5";
 		break;
 	}
+	lsf = info.version != MPG123_1_0;
 
-	const char* layer = "unknown";
+	std::string layer = "unknown";
 	switch(info.layer) {
 	case 1:
 		layer = "I";
@@ -280,7 +285,7 @@ void MP2Decoder::ProcessFormat() {
 		break;
 	}
 
-	const char* mode = "unknown";
+	std::string mode = "unknown";
 	switch(info.mode) {
 	case MPG123_M_STEREO:
 		mode = "Stereo";
@@ -296,12 +301,12 @@ void MP2Decoder::ProcessFormat() {
 		break;
 	}
 
-	std::stringstream ss;
-	ss << "MPEG " << version << " Layer " << layer << ", ";
-	ss << (info.rate / 1000) << " kHz ";
-	ss << mode << " ";
-	ss << "@ " << info.bitrate << " kbit/s";
-	observer->FormatChange(ss.str());
+	AUDIO_SERVICE_FORMAT format;
+	format.codec = "MPEG " + version + " Layer " + layer;
+	format.samplerate_khz = info.rate / 1000;
+	format.mode = mode;
+	format.bitrate_kbps = info.bitrate;
+	observer->FormatChange(format);
 
-	observer->StartAudio(info.rate, info.mode != MPG123_M_MONO ? 2 : 1, true);
+	observer->StartAudio(info.rate, info.mode != MPG123_M_MONO ? 2 : 1, float32);
 }
