@@ -31,10 +31,12 @@ static void usage(const char* exe) {
 	fprint_dablin_banner(stderr);
 	fprintf(stderr, "Usage: %s [OPTIONS] [file]\n", exe);
 	fprintf(stderr, "  -h            Show this help\n"
+					"  -f <format>   Source format: \"%s\" (default), \"%s\"\n"
 					"  -d <binary>   Use DAB live source (using the mentioned binary)\n"
 					"  -D <type>     DAB live source type: \"%s\" (default), \"%s\"\n"
 					"  -c <ch>       Channel to be played (requires DAB live source)\n"
 					"  -l <label>    Label of the service to be played\n"
+					"  -1            Play the first service found\n"
 					"  -s <sid>      ID of the service to be played\n"
 					"  -x <scids>    ID of the service component to be played (requires service ID)\n"
 					"  -r <subchid>  ID of the sub-channel (DAB) to be played\n"
@@ -45,6 +47,8 @@ static void usage(const char* exe) {
 					"  -I            Don't catch up on stream after interruption\n"
 					"  -F            Disable dynamic FIC messages (e.g. dynamic PTY)\n"
 					"  file          Input file to be played (stdin, if not specified)\n",
+					EnsembleSource::FORMAT_ETI.c_str(),
+					EnsembleSource::FORMAT_EDI.c_str(),
 					DABLiveETISource::TYPE_DAB2ETI.c_str(),
 					DABLiveETISource::TYPE_ETI_CMDLINE.c_str()
 			);
@@ -68,11 +72,14 @@ int main(int argc, char **argv) {
 
 	// option args
 	int c;
-	while((c = getopt(argc, argv, "hc:l:d:D:g:s:x:puIFr:R:")) != -1) {
+	while((c = getopt(argc, argv, "hf:c:l:d:D:g:s:x:1puIFr:R:")) != -1) {
 		switch(c) {
 		case 'h':
 			usage(argv[0]);
 			break;
+		case 'f':
+			options.source_format = optarg;
+			break;
 		case 'd':
 			options.dab_live_source_binary = optarg;
 			break;
@@ -86,6 +93,10 @@ int main(int argc, char **argv) {
 			options.initial_label = optarg;
 			initial_param_count++;
 			break;
+		case '1':
+			options.initial_first_found_service = true;
+			initial_param_count++;
+			break;
 		case 's':
 			options.initial_sid = strtol(optarg, nullptr, 0);
 			initial_param_count++;
@@ -140,6 +151,10 @@ int main(int argc, char **argv) {
 			usage(argv[0]);
 		}
 	} else {
+		if(options.source_format != EnsembleSource::FORMAT_ETI) {
+			fprintf(stderr, "A DAB live source can only be used with ETI source format!\n");
+			usage(argv[0]);
+		}
 		if(!options.filename.empty()) {
 			fprintf(stderr, "Both a file and DAB live source cannot be used as source!\n");
 			usage(argv[0]);
@@ -199,31 +214,38 @@ DABlinText::DABlinText(DABlinTextOptions options) {
 	// set XTerm window title to version string
 	fprintf(stderr, "\x1B]0;" "DABlin v" DABLIN_VERSION "\a");
 
-	eti_player = new ETIPlayer(options.pcm_output, options.untouched_output, options.disable_int_catch_up, this);
+	if(options.source_format == EnsembleSource::FORMAT_ETI)
+		ensemble_player = new ETIPlayer(options.pcm_output, options.untouched_output, options.disable_int_catch_up, this);
+	else
+		ensemble_player = new EDIPlayer(options.pcm_output, options.untouched_output, options.disable_int_catch_up, this);
 
 	// set initial sub-channel, if desired
 	if(options.initial_subchid_dab != AUDIO_SERVICE::subchid_none) {
-		eti_player->SetAudioService(AUDIO_SERVICE(options.initial_subchid_dab, false));
+		ensemble_player->SetAudioService(AUDIO_SERVICE(options.initial_subchid_dab, false));
 
 		// set XTerm window title to sub-channel number
 		fprintf(stderr, "\x1B]0;" "Sub-channel %d (DAB) - DABlin" "\a", options.initial_subchid_dab);
 	}
 	if(options.initial_subchid_dab_plus != AUDIO_SERVICE::subchid_none) {
-		eti_player->SetAudioService(AUDIO_SERVICE(options.initial_subchid_dab_plus, true));
+		ensemble_player->SetAudioService(AUDIO_SERVICE(options.initial_subchid_dab_plus, true));
 
 		// set XTerm window title to sub-channel number
 		fprintf(stderr, "\x1B]0;" "Sub-channel %d (DAB+) - DABlin" "\a", options.initial_subchid_dab_plus);
 	}
 
-	if(options.dab_live_source_binary.empty()) {
-		eti_source = new ETISource(options.filename, this);
-	} else {
-		DAB_LIVE_SOURCE_CHANNEL channel(options.initial_channel, dab_channels.at(options.initial_channel), options.gain);
+	if(options.source_format == EnsembleSource::FORMAT_ETI) {
+		if(options.dab_live_source_binary.empty()) {
+			ensemble_source = new ETISource(options.filename, this);
+		} else {
+			DAB_LIVE_SOURCE_CHANNEL channel(options.initial_channel, dab_channels.at(options.initial_channel), options.gain);
 
-		if(options.dab_live_source_type == DABLiveETISource::TYPE_ETI_CMDLINE)
-			eti_source = new EtiCmdlineETISource(options.dab_live_source_binary, channel, this);
-		else
-			eti_source = new DAB2ETIETISource(options.dab_live_source_binary, channel, this);
+			if(options.dab_live_source_type == DABLiveETISource::TYPE_ETI_CMDLINE)
+				ensemble_source = new EtiCmdlineETISource(options.dab_live_source_binary, channel, this);
+			else
+				ensemble_source = new DAB2ETIETISource(options.dab_live_source_binary, channel, this);
+		}
+	} else {
+		ensemble_source = new EDISource(options.filename, this);
 	}
 
 	fic_decoder = new FICDecoder(this, options.disable_dyn_fic_msgs);
@@ -231,12 +253,12 @@ DABlinText::DABlinText(DABlinTextOptions options) {
 
 DABlinText::~DABlinText() {
 	DoExit();
-	delete eti_source;
-	delete eti_player;
+	delete ensemble_source;
+	delete ensemble_player;
 	delete fic_decoder;
 }
 
-void DABlinText::ETIUpdateProgress(const ETI_PROGRESS& progress) {
+void DABlinText::EnsembleUpdateProgress(const ENSEMBLE_PROGRESS& progress) {
 	// compensate cursor movement
 	std::string format = "\x1B[34m" "%s" "\x1B[0m";
 	format.append(progress.text.length(), '\b');
@@ -249,13 +271,20 @@ void DABlinText::FICChangeService(const LISTED_SERVICE& service) {
 
 	std::string label = FICDecoder::ConvertLabelToUTF8(service.label, nullptr);
 
+	// if first found service requested, adopt service params (for possible later changes)
+	if(options.initial_first_found_service) {
+		options.initial_sid = service.sid;
+		options.initial_scids = service.scids;
+		options.initial_first_found_service = false;
+	}
+
 	// abort, if no/not initial service
 	if(!(label == options.initial_label || (service.sid == options.initial_sid && service.scids == options.initial_scids)))
 		return;
 
 	// if the audio service changed, switch
-	if(!eti_player->IsSameAudioService(service.audio_service))
-		eti_player->SetAudioService(service.audio_service);
+	if(!ensemble_player->IsSameAudioService(service.audio_service))
+		ensemble_player->SetAudioService(service.audio_service);
 
 	// set XTerm window title to service name
 	fprintf(stderr, "\x1B]0;" "%s - DABlin" "\a", label.c_str());
