@@ -33,12 +33,14 @@ static void usage(const char* exe) {
 	fprint_dablin_banner(stderr);
 	fprintf(stderr, "Usage: %s [OPTIONS] [file]\n", exe);
 	fprintf(stderr, "  -h           Show this help\n"
+					"  -f <format>  Source format: \"%s\" (default), \"%s\"\n"
 					"  -d <binary>  Use DAB live source (using the mentioned binary)\n"
 					"  -D <type>    DAB live source type: \"%s\" (default), \"%s\"\n"
 					"  -C <ch>,...  Channels to be listed (comma separated; requires DAB live source;\n"
 					"               an optional gain can also be specified, e.g. \"5C:-54\")\n"
 					"  -c <ch>      Channel to be played (requires DAB live source)\n"
 					"  -l <label>   Label of the service to be played\n"
+					"  -1           Play the first service found\n"
 					"  -s <sid>     ID of the service to be played\n"
 					"  -x <scids>   ID of the service component to be played (requires service ID)\n"
 					"  -g <gain>    USB stick gain to pass to DAB live source (auto gain is default)\n"
@@ -51,6 +53,8 @@ static void usage(const char* exe) {
 					"  -L           Enable loose behaviour (e.g. PAD conformance)\n"
 					"  -F           Disable dynamic FIC messages (e.g. dynamic PTY)\n"
 					"  file         Input file to be played (stdin, if not specified)\n",
+					EnsembleSource::FORMAT_ETI.c_str(),
+					EnsembleSource::FORMAT_EDI.c_str(),
 					DABLiveETISource::TYPE_DAB2ETI.c_str(),
 					DABLiveETISource::TYPE_ETI_CMDLINE.c_str(),
 					options_default.recordings_path.c_str(),
@@ -76,11 +80,14 @@ int main(int argc, char **argv) {
 
 	// option args
 	int c;
-	while((c = getopt(argc, argv, "hd:D:C:c:l:g:r:P:s:x:puISLF")) != -1) {
+	while((c = getopt(argc, argv, "hf:d:D:C:c:l:g:r:P:s:x:1puISLF")) != -1) {
 		switch(c) {
 		case 'h':
 			usage(argv[0]);
 			break;
+		case 'f':
+			options.source_format = optarg;
+			break;
 		case 'd':
 			options.dab_live_source_binary = optarg;
 			break;
@@ -97,6 +104,10 @@ int main(int argc, char **argv) {
 			options.initial_label = optarg;
 			initial_param_count++;
 			break;
+		case '1':
+			options.initial_first_found_service = true;
+			initial_param_count++;
+			break;
 		case 's':
 			options.initial_sid = strtol(optarg, nullptr, 0);
 			initial_param_count++;
@@ -159,6 +170,10 @@ int main(int argc, char **argv) {
 			usage(argv[0]);
 		}
 	} else {
+		if(options.source_format != EnsembleSource::FORMAT_ETI) {
+			fprintf(stderr, "A DAB live source can only be used with ETI source format!\n");
+			usage(argv[0]);
+		}
 		if(!options.filename.empty()) {
 			fprintf(stderr, "Both a file and DAB live source cannot be used as source!\n");
 			usage(argv[0]);
@@ -214,27 +229,40 @@ int main(int argc, char **argv) {
 DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	this->options = options;
 
-	initial_channel_appended = false;
+	switch_service_label = options.initial_label;
+	switch_service_sid = options.initial_sid;
+	switch_service_scids = options.initial_scids;
+	switch_service_applied = false;
+
 	rec_file = nullptr;
 	rec_duration_ms = 0;
 	rec_prebuffer_filled_ms = 0;
 
 	slideshow_window.set_transient_for(*this);
 
-	eti_update_progress.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::ETIUpdateProgressEmitted));
-	eti_change_format.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::ETIChangeFormatEmitted));
+	ensemble_update_progress.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::EnsembleUpdateProgressEmitted));
+	ensemble_change_format.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::EnsembleChangeFormatEmitted));
 	fic_change_ensemble.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::FICChangeEnsembleEmitted));
 	fic_change_service.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::FICChangeServiceEmitted));
 	pad_change_dynamic_label.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeDynamicLabelEmitted));
 	pad_change_slide.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeSlideEmitted));
+	do_rec_status_update.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::DoRecStatusUpdateEmitted));
 
-	eti_player = new ETIPlayer(options.pcm_output, options.untouched_output, options.disable_int_catch_up, this);
+	if(options.source_format == EnsembleSource::FORMAT_ETI)
+		ensemble_player = new ETIPlayer(options.pcm_output, options.untouched_output, options.disable_int_catch_up, this);
+	else
+		ensemble_player = new EDIPlayer(options.pcm_output, options.untouched_output, options.disable_int_catch_up, this);
 
-	if(!options.dab_live_source_binary.empty()) {
-		eti_source = nullptr;
+	if(options.source_format == EnsembleSource::FORMAT_ETI) {
+		if(!options.dab_live_source_binary.empty()) {
+			ensemble_source = nullptr;
+		} else {
+			ensemble_source = new ETISource(options.filename, this);
+			ensemble_source_thread = std::thread(&EnsembleSource::Main, ensemble_source);
+		}
 	} else {
-		eti_source = new ETISource(options.filename, this);
-		eti_source_thread = std::thread(&ETISource::Main, eti_source);
+		ensemble_source = new EDISource(options.filename, this);
+		ensemble_source_thread = std::thread(&EnsembleSource::Main, ensemble_source);
 	}
 
 	fic_decoder = new FICDecoder(this, options.disable_dyn_fic_msgs);
@@ -250,7 +278,6 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	// combobox first must be visible
 	if(combo_channels_liststore->iter_is_valid(initial_channel_it))
 		combo_channels.set_active(initial_channel_it);
-	initial_channel_appended = true;
 
 	// add window key press event handler
 	signal_key_press_event().connect(sigc::mem_fun(*this, &DABlinGTK::HandleKeyPressEvent));
@@ -262,13 +289,13 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 }
 
 DABlinGTK::~DABlinGTK() {
-	if(eti_source) {
-		eti_source->DoExit();
-		eti_source_thread.join();
-		delete eti_source;
+	if(ensemble_source) {
+		ensemble_source->DoExit();
+		ensemble_source_thread.join();
+		delete ensemble_source;
 	}
 
-	delete eti_player;
+	delete ensemble_player;
 
 	delete pad_decoder;
 	delete fic_decoder;
@@ -301,8 +328,11 @@ void DABlinGTK::InitWidgets() {
 
 	// init widgets
 	frame_combo_channels.set_label("Channel");
-	frame_combo_channels.set_size_request(75, -1);
-	frame_combo_channels.add(combo_channels);
+	frame_combo_channels.set_size_request(100, -1);
+	frame_combo_channels.add(channels_box);
+
+	channels_box.pack_start(combo_channels);
+	channels_box.pack_start(btn_channels_stop, Gtk::PACK_SHRINK);
 
 	combo_channels_liststore = Gtk::ListStore::create(combo_channels_cols);
 	combo_channels_liststore->set_default_sort_func(sigc::mem_fun(*this, &DABlinGTK::ComboChannelsSlotCompare));
@@ -317,6 +347,11 @@ void DABlinGTK::InitWidgets() {
 	else
 		frame_combo_channels.set_sensitive(false);
 
+	btn_channels_stop.set_image_from_icon_name("media-playback-start");
+	btn_channels_stop.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_btn_channels_stop));
+	btn_channels_stop.set_sensitive(false);
+	btn_channels_stop.set_tooltip_text("Stop/resume decoding current channel/service");
+
 	frame_label_ensemble.set_label("Ensemble");
 	frame_label_ensemble.set_size_request(150, -1);
 	frame_label_ensemble.add(label_ensemble);
@@ -347,17 +382,23 @@ void DABlinGTK::InitWidgets() {
 	tglbtn_record.set_image_from_icon_name("media-record");
 	tglbtn_record.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_record));
 	tglbtn_record.set_sensitive(false);
+	tglbtn_record.set_tooltip_text("Record service");
 
-	tglbtn_mute.set_label("Mute");
-	tglbtn_mute.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_mute));
+	label_record.set_halign(Gtk::ALIGN_START);
+	label_record.set_padding(WIDGET_SPACE, WIDGET_SPACE);
 
-	vlmbtn.set_value(1.0);
-	vlmbtn.set_sensitive(eti_player->HasAudioVolumeControl());
-	vlmbtn.signal_value_changed().connect(sigc::mem_fun(*this, &DABlinGTK::on_vlmbtn));
-
-	tglbtn_slideshow.set_label("Slideshow");
+	tglbtn_slideshow.set_image_from_icon_name("video-display");
 	tglbtn_slideshow.set_active(!options.initially_disable_slideshow);
 	tglbtn_slideshow.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_slideshow));
+	tglbtn_slideshow.set_tooltip_text("Slideshow (if available)");
+
+	tglbtn_mute.set_image_from_icon_name("audio-volume-muted");
+	tglbtn_mute.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_mute));
+	tglbtn_mute.set_tooltip_text("Mute");
+
+	vlmbtn.set_value(1.0);
+	vlmbtn.set_sensitive(ensemble_player->HasAudioVolumeControl());
+	vlmbtn.signal_value_changed().connect(sigc::mem_fun(*this, &DABlinGTK::on_vlmbtn));
 
 	frame_label_dl.set_label("Dynamic Label");
 	frame_label_dl.set_size_request(750, 50);
@@ -384,9 +425,10 @@ void DABlinGTK::InitWidgets() {
 	top_grid.attach_next_to(frame_combo_services, frame_label_ensemble, Gtk::POS_RIGHT, 1, 2);
 	top_grid.attach_next_to(frame_label_format, frame_combo_services, Gtk::POS_RIGHT, 1, 2);
 	top_grid.attach_next_to(tglbtn_record, frame_label_format, Gtk::POS_RIGHT, 1, 1);
-	top_grid.attach_next_to(tglbtn_mute, tglbtn_record, Gtk::POS_RIGHT, 1, 1);
+	top_grid.attach_next_to(label_record, tglbtn_record, Gtk::POS_RIGHT, 2, 1);
+	top_grid.attach_next_to(tglbtn_slideshow, tglbtn_record, Gtk::POS_BOTTOM, 1, 1);
+	top_grid.attach_next_to(tglbtn_mute, tglbtn_slideshow, Gtk::POS_RIGHT, 1, 1);
 	top_grid.attach_next_to(vlmbtn, tglbtn_mute, Gtk::POS_RIGHT, 1, 1);
-	top_grid.attach_next_to(tglbtn_slideshow, tglbtn_record, Gtk::POS_BOTTOM, 3, 1);
 	top_grid.attach_next_to(frame_label_dl, frame_combo_channels, Gtk::POS_BOTTOM, 7, 1);
 	top_grid.attach_next_to(progress_position, frame_label_dl, Gtk::POS_BOTTOM, 7, 1);
 
@@ -486,11 +528,11 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 	}
 
 	// if the audio service changed, reset format/DL/slide + switch
-	if(!eti_player->IsSameAudioService(service.audio_service)) {
+	if(!ensemble_player->IsSameAudioService(service.audio_service)) {
 		// stop playback of old service
 		if(options.rec_prebuffer_size_s > 0)
-			eti_player->RemoveUntouchedStreamConsumer(this);
-		eti_player->SetAudioService(AUDIO_SERVICE());
+			ensemble_player->RemoveUntouchedStreamConsumer(this);
+		ensemble_player->SetAudioService(AUDIO_SERVICE());
 
 		pad_decoder->Reset();
 
@@ -505,18 +547,22 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 			slideshow_window.ClearSlide();
 		}
 
-		if(options.rec_prebuffer_size_s > 0) {
+		{
 			std::lock_guard<std::mutex> lock(rec_mutex);
 
-			// clear prebuffer
-			rec_prebuffer.clear();
-			rec_prebuffer_filled_ms = 0;
+			// clear prebuffer, if enabled
+			if(options.rec_prebuffer_size_s > 0) {
+				rec_prebuffer.clear();
+				rec_prebuffer_filled_ms = 0;
+			}
+
+			UpdateRecStatus(!service.IsNone());
 		}
 
 		// start playback of new service
-		eti_player->SetAudioService(service.audio_service);
+		ensemble_player->SetAudioService(service.audio_service);
 		if(options.rec_prebuffer_size_s > 0)
-			eti_player->AddUntouchedStreamConsumer(this);
+			ensemble_player->AddUntouchedStreamConsumer(this);
 	}
 
 	if(service.HasSLS()) {
@@ -530,15 +576,25 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 
 void DABlinGTK::on_tglbtn_record() {
 	if(tglbtn_record.get_active()) {
-		// get timestamp
-		const time_t now = time(nullptr);
-		if(now == (time_t) -1)
-			perror("DABlinGTK: error while getting time");
-		struct tm* now_tm = localtime(&now);
-		if(!now_tm)
+		// get start timestamp
+		time_t start;
+		{
+			std::lock_guard<std::mutex> lock(rec_mutex);
+
+			// use prebuffer start information, if available
+			if(!rec_prebuffer.empty()) {
+				start = rec_prebuffer.front().ts;
+			} else {
+				start = time(nullptr);
+				if(start == (time_t) -1)
+					perror("DABlinGTK: error while getting time for recording");
+			}
+		}
+		struct tm* start_tm = localtime(&start);
+		if(!start_tm)
 			perror("DABlinGTK: error while getting local time");
-		char now_string[22];
-		strftime(now_string, sizeof(now_string), "%F - %H-%M-%S", now_tm);
+		char start_string[22];
+		strftime(start_string, sizeof(start_string), "%F - %H-%M-%S", start_tm);
 
 		// get station name
 		LISTED_SERVICE service;	// default: none
@@ -555,11 +611,12 @@ void DABlinGTK::on_tglbtn_record() {
 			if(c == '/')
 				c = '_';
 
-		std::string new_rec_filename = options.recordings_path + "/" + std::string(now_string) + " - " + label_cleaned + "." + eti_player->GetUntouchedStreamFileExtension();
+		std::string new_rec_filename = options.recordings_path + "/" + std::string(start_string) + " - " + label_cleaned + "." + ensemble_player->GetUntouchedStreamFileExtension();
 		FILE* new_rec_file = fopen(new_rec_filename.c_str(), "wb");
 		if(new_rec_file) {
 			// disable channel/service switch
 			combo_channels.set_sensitive(false);	// parent frame already non-sensitive, if channels not available
+			btn_channels_stop.set_sensitive(false);	// parent frame already non-sensitive, if channels not available
 			combo_services.set_sensitive(false);
 
 			set_icon_name("media-record");
@@ -581,11 +638,11 @@ void DABlinGTK::on_tglbtn_record() {
 				rec_prebuffer.clear();
 				rec_prebuffer_filled_ms = 0;
 
-				UpdateRecStatus();
+				UpdateRecStatus(true);
 			}
 
 			if(options.rec_prebuffer_size_s == 0)
-				eti_player->AddUntouchedStreamConsumer(this);
+				ensemble_player->AddUntouchedStreamConsumer(this);
 		} else {
 			int fopen_errno = errno;	// save before overwrite
 			tglbtn_record.set_active(false);
@@ -601,17 +658,18 @@ void DABlinGTK::on_tglbtn_record() {
 		// only stop recording, if active (handles error on recording start)
 		if(rec_file) {
 			if(options.rec_prebuffer_size_s == 0)
-				eti_player->RemoveUntouchedStreamConsumer(this);
+				ensemble_player->RemoveUntouchedStreamConsumer(this);
 
 			fclose(rec_file);
 			rec_file = nullptr;
 
 			set_icon_name("media-playback-start");
 			fprintf(stderr, "DABlinGTK: recording stopped\n");
-			UpdateRecStatus();
+			UpdateRecStatus(true);
 
 			// enable channel/service switch
 			combo_channels.set_sensitive(true);		// parent frame already non-sensitive, if channels not available
+			btn_channels_stop.set_sensitive(true);	// parent frame already non-sensitive, if channels not available
 			combo_services.set_sensitive(true);
 		}
 	}
@@ -631,7 +689,7 @@ void DABlinGTK::ProcessUntouchedStream(const uint8_t* data, size_t len, size_t d
 
 		// update status only on seconds change
 		if(rec_duration_ms / 1000 != rec_duration_ms_old / 1000)
-			UpdateRecStatus();
+			DoRecStatusUpdate(true);
 	} else {
 		// if prebuffer enabled
 		if(options.rec_prebuffer_size_s > 0) {
@@ -649,37 +707,44 @@ void DABlinGTK::ProcessUntouchedStream(const uint8_t* data, size_t len, size_t d
 
 			// update status only on seconds change
 			if(rec_prebuffer_filled_ms / 1000 != rec_prebuffer_ms_old / 1000)
-				UpdateRecStatus();
+				DoRecStatusUpdate(true);
 		}
 	}
 }
 
-void DABlinGTK::UpdateRecStatus() {
-	// mutex must already be locked!
+void DABlinGTK::DoRecStatusUpdateEmitted() {
+	std::lock_guard<std::mutex> lock(rec_mutex);
+	UpdateRecStatus(do_rec_status_update.Pop());
+}
 
-	std::string tooltip_text;
+void DABlinGTK::UpdateRecStatus(bool decoding) {
+	// mutex must already be locked!
 
 	// if recording in progress
 	if(rec_file) {
-		tooltip_text =
-				"File name: \"" + rec_filename + "\"\n"
-				"Duration: " + StringTools::MsToTimecode(rec_duration_ms);
+		label_record.set_label(StringTools::MsToTimecode(rec_duration_ms));
+		label_record.set_sensitive(true);
+		label_record.set_tooltip_text("File name: \"" + rec_filename + "\"");
 	} else {
-		// if prebuffer enabled
-		if(rec_prebuffer_filled_ms > 0)
-			tooltip_text = "Prebuffer: " + StringTools::MsToTimecode(rec_prebuffer_filled_ms) + " / " + StringTools::MsToTimecode(options.rec_prebuffer_size_s * 1000);
+		// if prebuffer enabled and currently decoding
+		if(options.rec_prebuffer_size_s > 0 && decoding) {
+			label_record.set_label(StringTools::MsToTimecode(rec_prebuffer_filled_ms));
+			label_record.set_sensitive(false);
+			label_record.set_tooltip_text("Prebuffer: " + StringTools::MsToTimecode(options.rec_prebuffer_size_s * 1000));
+		} else {
+			label_record.set_label("");
+			label_record.set_tooltip_text("");
+		}
 	}
-
-	tglbtn_record.set_tooltip_text(tooltip_text);
 }
 
 
 void DABlinGTK::on_tglbtn_mute() {
-	eti_player->SetAudioMute(tglbtn_mute.get_active());
+	ensemble_player->SetAudioMute(tglbtn_mute.get_active());
 }
 
 void DABlinGTK::on_vlmbtn(double value) {
-	eti_player->SetAudioVolume(value);
+	ensemble_player->SetAudioVolume(value);
 
 	// disable mute, if needed
 	if(tglbtn_mute.get_active())
@@ -752,11 +817,16 @@ bool DABlinGTK::HandleKeyPressEvent(GdkEventKey* key_event) {
 
 		switch(key_event->keyval) {
 		case GDK_KEY_c:
-		case GDK_KEY_C:
+		case GDK_KEY_C: {
 			// copy DL to clipboard, if not empty
 			std::string dl = label_dl.get_label();
 			if(!dl.empty())
 				Gtk::Clipboard::get()->set_text(dl);
+			return true; }
+		case GDK_KEY_space:
+			// stop/resume decoding the current channel/service, if allowed
+			if(btn_channels_stop.get_sensitive())
+				btn_channels_stop.clicked();
 			return true;
 		}
 	}
@@ -845,10 +915,10 @@ bool DABlinGTK::HandleConfigureEvent(GdkEventConfigure* /*configure_event*/) {
 	return false;
 }
 
-void DABlinGTK::ETIUpdateProgressEmitted() {
-//	fprintf(stderr, "### ETIUpdateProgressEmitted\n");
+void DABlinGTK::EnsembleUpdateProgressEmitted() {
+//	fprintf(stderr, "### EnsembleUpdateProgressEmitted\n");
 
-	ETI_PROGRESS progress = eti_update_progress.Pop();
+	ENSEMBLE_PROGRESS progress = ensemble_update_progress.Pop();
 
 	progress_position.set_fraction(progress.value);
 	progress_position.set_text(progress.text);
@@ -856,10 +926,10 @@ void DABlinGTK::ETIUpdateProgressEmitted() {
 		progress_position.show();
 }
 
-void DABlinGTK::ETIChangeFormatEmitted() {
-//	fprintf(stderr, "### ETIChangeFormatEmitted\n");
+void DABlinGTK::EnsembleChangeFormatEmitted() {
+//	fprintf(stderr, "### EnsembleChangeFormatEmitted\n");
 
-	label_format.set_label(eti_change_format.Pop().GetSummary());
+	label_format.set_label(ensemble_change_format.Pop().GetSummary());
 }
 
 void DABlinGTK::FICChangeEnsembleEmitted() {
@@ -911,8 +981,15 @@ void DABlinGTK::FICChangeServiceEmitted() {
 	row[combo_services_cols.col_service] = new_service;
 
 	if(add_new_row) {
+		// if first found service requested, adopt service params (for possible later changes)
+		if(options.initial_first_found_service) {
+			switch_service_sid = new_service.sid;
+			switch_service_scids = new_service.scids;
+			options.initial_first_found_service = false;
+		}
+
 		// set (initial) service
-		if(label == options.initial_label || (new_service.sid == options.initial_sid && new_service.scids == options.initial_scids))
+		if(label == switch_service_label || (new_service.sid == switch_service_sid && new_service.scids == switch_service_scids))
 			combo_services.set_active(row_it);
 	} else {
 		// set (updated) service
@@ -927,38 +1004,80 @@ void DABlinGTK::FICDiscardedFIB() {
 }
 
 void DABlinGTK::on_combo_channels() {
-	Gtk::TreeModel::Row row = *combo_channels.get_active();
-	DAB_LIVE_SOURCE_CHANNEL channel = row[combo_channels_cols.col_channel];
+	btn_channels_stop.set_sensitive(true);
 
 	// cleanup
-	if(eti_source) {
-		eti_source->DoExit();
-		eti_source_thread.join();
-		delete eti_source;
+	if(ensemble_source) {
+		ensemble_source->DoExit();
+		ensemble_source_thread.join();
+		delete ensemble_source;
+		ensemble_source = nullptr;
 	}
 
-	ETIResetFIC();
+	EnsembleResetFIC();
 	combo_services_liststore->clear();	// TODO: prevent on_combo_services() being called for each deleted row
+	ensemble_player->StopAudio();
 	label_ensemble.set_label("");
 	frame_label_ensemble.set_tooltip_text("");
-	frame_combo_channels.set_tooltip_text(
-			"Center frequency: " + std::to_string(channel.freq) + " kHz\n"
-			"Gain: " + channel.GainToString()
-	);
 
-	// prevent re-use of initial params
-	if(initial_channel_appended) {
-		options.initial_label = "";
-		options.initial_sid = LISTED_SERVICE::sid_none;
-		options.initial_scids = LISTED_SERVICE::scids_none;
+	// apply
+	Gtk::TreeModel::iterator row_it = combo_channels.get_active();
+	if(combo_channels_liststore->iter_is_valid(row_it)) {
+		Gtk::TreeModel::Row row = *row_it;
+		DAB_LIVE_SOURCE_CHANNEL channel = row[combo_channels_cols.col_channel];
+
+		combo_channels.set_tooltip_text(
+				"Center frequency: " + std::to_string(channel.freq) + " kHz\n"
+				"Gain: " + channel.GainToString()
+		);
+
+		// prevent re-use of switch service params next time
+		if(!switch_service_applied) {
+			switch_service_applied = true;
+		} else {
+			switch_service_label = "";
+			switch_service_sid = LISTED_SERVICE::sid_none;
+			switch_service_scids = LISTED_SERVICE::scids_none;
+		}
+
+		if(options.dab_live_source_type == DABLiveETISource::TYPE_ETI_CMDLINE)
+			ensemble_source = new EtiCmdlineETISource(options.dab_live_source_binary, channel, this);
+		else
+			ensemble_source = new DAB2ETIETISource(options.dab_live_source_binary, channel, this);
+		ensemble_source_thread = std::thread(&EnsembleSource::Main, ensemble_source);
+
+		btn_channels_stop.set_image_from_icon_name("media-playback-stop");
+	} else {
+		combo_channels.set_tooltip_text("");
+		btn_channels_stop.set_image_from_icon_name("media-playback-start");
 	}
+}
 
-	// append
-	if(options.dab_live_source_type == DABLiveETISource::TYPE_ETI_CMDLINE)
-		eti_source = new EtiCmdlineETISource(options.dab_live_source_binary, channel, this);
-	else
-		eti_source = new DAB2ETIETISource(options.dab_live_source_binary, channel, this);
-	eti_source_thread = std::thread(&ETISource::Main, eti_source);
+void DABlinGTK::on_btn_channels_stop() {
+	if(combo_channels.get_active_row_number() != -1) {
+		// save state + stop
+		Gtk::TreeModel::iterator row_it = combo_services.get_active();
+		if(combo_services_liststore->iter_is_valid(row_it)) {
+			Gtk::TreeModel::Row row = *row_it;
+			LISTED_SERVICE service = row[combo_services_cols.col_service];
+			resume_service_sid = service.sid;
+			resume_service_scids = service.scids;
+		} else {
+			resume_service_sid = LISTED_SERVICE::sid_none;
+			resume_service_scids = LISTED_SERVICE::scids_none;
+		}
+
+		resume_channel_it = combo_channels.get_active();
+		combo_channels.unset_active();
+	} else {
+		// load state + resume
+		switch_service_label = "";
+		switch_service_sid = resume_service_sid;
+		switch_service_scids = resume_service_scids;
+		switch_service_applied = false;
+
+		combo_channels.set_active(resume_channel_it);
+	}
 }
 
 void DABlinGTK::on_combo_services() {
