@@ -33,6 +33,8 @@
 #include "dablin_gtk_sls.h"
 #include "eti_source.h"
 #include "eti_player.h"
+#include "edi_source.h"
+#include "edi_player.h"
 #include "fic_decoder.h"
 #include "pad_decoder.h"
 #include "tools.h"
@@ -71,6 +73,8 @@ public:
 // --- DABlinGTKOptions -----------------------------------------------------------------
 struct DABlinGTKOptions {
 	std::string filename;
+	std::string source_format;
+	bool initial_first_found_service;
 	std::string initial_label;
 	int initial_sid;
 	int initial_scids;
@@ -89,6 +93,8 @@ struct DABlinGTKOptions {
 	bool disable_dyn_fic_msgs;
 	
 DABlinGTKOptions() :
+	source_format(EnsembleSource::FORMAT_ETI),
+	initial_first_found_service(false),
 	initial_sid(LISTED_SERVICE::sid_none),
 	initial_scids(LISTED_SERVICE::scids_none),
 	dab_live_source_type(DABLiveETISource::TYPE_DAB2ETI),
@@ -136,11 +142,17 @@ public:
 // --- RecSample -----------------------------------------------------------------
 struct RecSample {
 	std::vector<uint8_t> data;
+	time_t ts;
 	size_t duration_ms;
 
 	RecSample(const uint8_t* data, size_t len, size_t duration_ms) {
 		this->data.resize(len);
 		memcpy(&this->data[0], data, len);
+
+		ts = time(nullptr);
+		if(ts == (time_t) -1)
+			perror("DABlinGTK: error while getting time for sample");
+
 		this->duration_ms = duration_ms;
 	}
 };
@@ -149,19 +161,23 @@ typedef std::list<RecSample> rec_samples_t;
 
 
 // --- DABlinGTK -----------------------------------------------------------------
-class DABlinGTK : public Gtk::Window, ETISourceObserver, ETIPlayerObserver, FICDecoderObserver, PADDecoderObserver, UntouchedStreamConsumer {
+class DABlinGTK : public Gtk::Window, EnsembleSourceObserver, EnsemblePlayerObserver, FICDecoderObserver, PADDecoderObserver, UntouchedStreamConsumer {
 private:
 	DABlinGTKOptions options;
 
 	Gtk::ListStore::iterator initial_channel_it;
-	bool initial_channel_appended;
+
+	std::string switch_service_label;
+	int switch_service_sid;
+	int switch_service_scids;
+	bool switch_service_applied;
 
 	DABlinGTKSlideshowWindow slideshow_window;
 
-	ETISource *eti_source;
-	std::thread eti_source_thread;
+	EnsembleSource *ensemble_source;
+	std::thread ensemble_source_thread;
 
-	ETIPlayer *eti_player;
+	EnsemblePlayer *ensemble_player;
 
 	FICDecoder *fic_decoder;
 	PADDecoder *pad_decoder;
@@ -173,19 +189,24 @@ private:
 	rec_samples_t rec_prebuffer;
 	long int rec_prebuffer_filled_ms;
 
-	// ETI data change
-	GTKDispatcherQueue<ETI_PROGRESS> eti_update_progress;
-	void ETIProcessFrame(const uint8_t *data) {eti_player->ProcessFrame(data);}
-	void ETIUpdateProgress(const ETI_PROGRESS& progress) {eti_update_progress.PushAndEmit(progress);}
-	void ETIUpdateProgressEmitted();
+	Gtk::TreeModel::iterator resume_channel_it;
+	int resume_service_sid;
+	int resume_service_scids;
 
-	GTKDispatcherQueue<AUDIO_SERVICE_FORMAT> eti_change_format;
-	void ETIChangeFormat(const AUDIO_SERVICE_FORMAT& format) {eti_change_format.PushAndEmit(format);}
-	void ETIChangeFormatEmitted();
+	// ensemble progress change
+	GTKDispatcherQueue<ENSEMBLE_PROGRESS> ensemble_update_progress;
+	void EnsembleProcessFrame(const uint8_t *data) {ensemble_player->ProcessFrame(data);}
+	void EnsembleUpdateProgress(const ENSEMBLE_PROGRESS& progress) {ensemble_update_progress.PushAndEmit(progress);}
+	void EnsembleUpdateProgressEmitted();
 
-	void ETIProcessFIC(const uint8_t *data, size_t len) {fic_decoder->Process(data, len);}
-	void ETIResetFIC() {fic_decoder->Reset();}
-	void ETIProcessPAD(const uint8_t *xpad_data, size_t xpad_len, bool exact_xpad_len, const uint8_t* fpad_data) {pad_decoder->Process(xpad_data, xpad_len, exact_xpad_len, fpad_data);}
+	// ensemble data change
+	GTKDispatcherQueue<AUDIO_SERVICE_FORMAT> ensemble_change_format;
+	void EnsembleChangeFormat(const AUDIO_SERVICE_FORMAT& format) {ensemble_change_format.PushAndEmit(format);}
+	void EnsembleChangeFormatEmitted();
+
+	void EnsembleProcessFIC(const uint8_t *data, size_t len) {fic_decoder->Process(data, len);}
+	void EnsembleResetFIC() {fic_decoder->Reset();}
+	void EnsembleProcessPAD(const uint8_t *xpad_data, size_t xpad_len, bool exact_xpad_len, const uint8_t* fpad_data) {pad_decoder->Process(xpad_data, xpad_len, exact_xpad_len, fpad_data);}
 
 	void ProcessUntouchedStream(const uint8_t* data, size_t len, size_t duration_ms);
 
@@ -193,10 +214,12 @@ private:
 	Gtk::Grid top_grid;
 
 	Gtk::Frame frame_combo_channels;
+	Gtk::Box channels_box;
 	DABlinGTKChannelColumns combo_channels_cols;
 	Glib::RefPtr<Gtk::ListStore> combo_channels_liststore;
 	Gtk::ComboBox combo_channels;
 	int ComboChannelsSlotCompare(const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b);
+	Gtk::Button btn_channels_stop;
 
 	FIC_ENSEMBLE ensemble;
 	Gtk::Frame frame_label_ensemble;
@@ -212,9 +235,11 @@ private:
 	Gtk::Label label_format;
 
 	Gtk::ToggleButton tglbtn_record;
+	Gtk::Label label_record;
+
+	Gtk::ToggleButton tglbtn_slideshow;
 	Gtk::ToggleButton tglbtn_mute;
 	Gtk::VolumeButton vlmbtn;
-	Gtk::ToggleButton tglbtn_slideshow;
 
 	Gtk::Frame frame_label_dl;
 	Gtk::Label label_dl;
@@ -228,6 +253,7 @@ private:
 
 	void SetService(const LISTED_SERVICE& service);
 
+	void on_btn_channels_stop();
 	void on_tglbtn_record();
 	void on_tglbtn_mute();
 	void on_vlmbtn(double value);
@@ -240,7 +266,11 @@ private:
 	bool HandleConfigureEvent(GdkEventConfigure* configure_event);
 	bool CheckForIndexKey(GdkEventKey* key_event, int old_index, int& new_index);
 	void TrySwitch(Gtk::ComboBox& combo, Glib::RefPtr<Gtk::ListStore>& combo_liststore, int index);
-	void UpdateRecStatus();
+
+	GTKDispatcherQueue<bool> do_rec_status_update;
+	void DoRecStatusUpdate(bool decoding) {do_rec_status_update.PushAndEmit(decoding);}
+	void DoRecStatusUpdateEmitted();
+	void UpdateRecStatus(bool decoding);
 
 	// FIC data change
 	GTKDispatcherQueue<FIC_ENSEMBLE> fic_change_ensemble;
