@@ -1,6 +1,6 @@
 /*
     DABlin - capital DAB experience
-    Copyright (C) 2015-2018 Stefan Pöschel
+    Copyright (C) 2015-2019 Stefan Pöschel
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -98,9 +98,15 @@ void FICDecoder::ProcessFIG0(const uint8_t *data, size_t len) {
 	case 8:
 		ProcessFIG0_8(data, len);
 		break;
+	case 9:
+		ProcessFIG0_9(data, len);
+		break;
 	case 13:
 		ProcessFIG0_13(data, len);
 		break;
+	case 17:
+		ProcessFIG0_17(data, len);
+		break;
 //	default:
 //		fprintf(stderr, "FICDecoder: received unsupported FIG 0/%d with %zu field bytes\n", header.extension, len);
 	}
@@ -297,6 +303,34 @@ void FICDecoder::ProcessFIG0_8(const uint8_t *data, size_t len) {
 	}
 }
 
+void FICDecoder::ProcessFIG0_9(const uint8_t *data, size_t len) {
+	// FIG 0/9 - Time and country identifier - Country, LTO and International table
+	// ensemble ECC and international table ID only
+
+	if(len < 3)
+		return;
+
+	FIC_ENSEMBLE new_ensemble = ensemble;
+	new_ensemble.ecc = data[1];
+	new_ensemble.inter_table_id = data[2];
+
+	if(ensemble != new_ensemble) {
+		ensemble = new_ensemble;
+
+		fprintf(stderr, "FICDecoder: ECC: 0x%02X, international table ID: 0x%02X (%s)\n",
+				ensemble.ecc, ensemble.inter_table_id, ConvertInterTableIDToString(ensemble.inter_table_id).c_str());
+
+		UpdateEnsemble();
+
+		// update services that changes may affect
+		for(const fic_services_t::value_type& service : services) {
+			const FIC_SERVICE& s = service.second;
+			if(s.pty_static != FIC_SERVICE::pty_none || s.pty_dynamic != FIC_SERVICE::pty_none)
+				UpdateService(s);
+		}
+	}
+}
+
 void FICDecoder::ProcessFIG0_13(const uint8_t *data, size_t len) {
 	// FIG 0/13 - User application information
 	// programme services only
@@ -337,6 +371,49 @@ void FICDecoder::ProcessFIG0_13(const uint8_t *data, size_t len) {
 	}
 }
 
+void FICDecoder::ProcessFIG0_17(const uint8_t *data, size_t len) {
+	// FIG 0/17 - Programme Type
+	// programme type only
+
+	// iterate through all services
+	for(size_t offset = 0; offset < len;) {
+		uint16_t sid = data[offset] << 8 | data[offset + 1];
+		bool sd = data[offset + 2] & 0x80;
+		bool l_flag = data[offset + 2] & 0x20;
+		bool cc_flag = data[offset + 2] & 0x10;
+		offset += 3;
+
+		// skip language, if present
+		if(l_flag)
+			offset++;
+
+		// programme type (international code)
+		int pty = data[offset] & 0x1F;
+		offset++;
+
+		// skip CC part, if present
+		if(cc_flag)
+			offset++;
+
+		FIC_SERVICE& service = GetService(sid);
+		int& current_pty = sd ? service.pty_dynamic : service.pty_static;
+		if(current_pty != pty) {
+			// suppress message, if dynamic FIC messages disabled and dynamic PTY not initally be set
+			bool show_msg = !(disable_dyn_msgs && sd && current_pty != FIC_SERVICE::pty_none);
+
+			current_pty = pty;
+
+			if(show_msg) {
+				// assuming international table ID 0x01 here!
+				fprintf(stderr, "FICDecoder: SId 0x%04X: programme type (%s): '%s'\n",
+						sid, sd ? "dynamic" : "static", ConvertPTYToString(pty, 0x01).c_str());
+			}
+
+			UpdateService(service);
+		}
+	}
+}
+
 void FICDecoder::ProcessFIG1(const uint8_t *data, size_t len) {
 	if(len < 1) {
 		fprintf(stderr, "FICDecoder: received empty FIG 1\n");
@@ -408,9 +485,11 @@ void FICDecoder::ProcessFIG1_0(uint16_t eid, const FIC_LABEL& label) {
 		ensemble.label = label;
 
 		std::string label_str = ConvertLabelToUTF8(label, nullptr);
-		fprintf(stderr, "FICDecoder: EId 0x%04X: ensemble label '" "\x1B[32m" "%s" "\x1B[0m" "'\n", eid, label_str.c_str());
+		std::string short_label_str = DeriveShortLabelUTF8(label_str, label.short_label_mask);
+		fprintf(stderr, "FICDecoder: EId 0x%04X: ensemble label '" "\x1B[32m" "%s" "\x1B[0m" "' ('" "\x1B[32m" "%s" "\x1B[0m" "')\n",
+				eid, label_str.c_str(), short_label_str.c_str());
 
-		observer->FICChangeEnsemble(ensemble);
+		UpdateEnsemble();
 	}
 }
 
@@ -420,7 +499,9 @@ void FICDecoder::ProcessFIG1_1(uint16_t sid, const FIC_LABEL& label) {
 		service.label = label;
 
 		std::string label_str = ConvertLabelToUTF8(label, nullptr);
-		fprintf(stderr, "FICDecoder: SId 0x%04X: programme service label '" "\x1B[32m" "%s" "\x1B[0m" "'\n", sid, label_str.c_str());
+		std::string short_label_str = DeriveShortLabelUTF8(label_str, label.short_label_mask);
+		fprintf(stderr, "FICDecoder: SId 0x%04X: programme service label '" "\x1B[32m" "%s" "\x1B[0m" "' ('" "\x1B[32m" "%s" "\x1B[0m" "')\n",
+				sid, label_str.c_str(), short_label_str.c_str());
 
 		UpdateService(service);
 	}
@@ -435,7 +516,9 @@ void FICDecoder::ProcessFIG1_4(uint16_t sid, int scids, const FIC_LABEL& label)
 		comp_label = label;
 
 		std::string label_str = ConvertLabelToUTF8(label, nullptr);
-		fprintf(stderr, "FICDecoder: SId 0x%04X, SCIdS %2d: service component label '" "\x1B[32m" "%s" "\x1B[0m" "'\n", sid, scids, label_str.c_str());
+		std::string short_label_str = DeriveShortLabelUTF8(label_str, label.short_label_mask);
+		fprintf(stderr, "FICDecoder: SId 0x%04X, SCIdS %2d: service component label '" "\x1B[32m" "%s" "\x1B[0m" "' ('" "\x1B[32m" "%s" "\x1B[0m" "')\n",
+				sid, scids, label_str.c_str(), short_label_str.c_str());
 
 		UpdateService(service);
 	}
@@ -488,6 +571,8 @@ void FICDecoder::UpdateListedService(const FIC_SERVICE& service, int scids, bool
 	ls.sid = service.sid;
 	ls.scids = scids;
 	ls.label = service.label;
+	ls.pty_static = service.pty_static;
+	ls.pty_dynamic = service.pty_dynamic;
 	ls.pri_comp_subchid = service.pri_comp_subchid;
 	ls.multi_comps = multi_comps;
 
@@ -549,8 +634,17 @@ int FICDecoder::GetSLSAppType(const ua_data_t& ua_data) {
 		return LISTED_SERVICE::sls_app_type_none;
 }
 
+void FICDecoder::UpdateEnsemble() {
+	// abort update, if label not yet present
+	if(ensemble.label.IsNone())
+		return;
+
+	// forward to observer
+	observer->FICChangeEnsemble(ensemble);
+}
+
 std::string FICDecoder::ConvertLabelToUTF8(const FIC_LABEL& label, std::string* charset_name) {
-	std::string result = ConvertTextToUTF8(label.label, sizeof(label.label), label.charset, false, charset_name);
+	std::string result = CharsetTools::ConvertTextToUTF8(label.label, sizeof(label.label), label.charset, false, charset_name);
 
 	// discard trailing spaces
 	size_t last_pos = result.find_last_not_of(' ');
@@ -560,103 +654,6 @@ std::string FICDecoder::ConvertLabelToUTF8(const FIC_LABEL& label, std::string*
 	return result;
 }
 
-std::string FICDecoder::ConvertTextToUTF8(const uint8_t *data, size_t len, int charset, bool mot, std::string* charset_name) {
-	// remove undesired chars
-	std::vector<uint8_t> cleaned_data;
-	for(size_t i = 0; i < len; i++) {
-		switch(data[i]) {
-		case 0x00:	// NULL
-		case 0x0A:	// PLB
-		case 0x0B:	// EoH
-		case 0x1F:	// PWB
-			continue;
-		default:
-			cleaned_data.push_back(data[i]);
-		}
-	}
-
-	// convert characters
-	if(charset == 0b0000) {			// EBU Latin based
-		if(charset_name)
-			*charset_name = "EBU Latin based";
-
-		std::string result;
-		for(const uint8_t& c : cleaned_data)
-			result += ConvertCharEBUToUTF8(c);
-		return result;
-	}
-	if(charset == 0b0100 && mot)	// ISO/IEC-8859-1 (MOT only)
-		return ConvertStringIconvToUTF8(cleaned_data, charset_name, "ISO-8859-1");
-	if(charset == 0b0110 && !mot)	// UCS-2 BE (DAB only)
-		return ConvertStringIconvToUTF8(cleaned_data, charset_name, "UCS-2BE");
-	if(charset == 0b1111) {			// UTF-8
-		if(charset_name)
-			*charset_name = "UTF-8";
-
-		return std::string((char*) &cleaned_data[0], cleaned_data.size());
-	}
-
-	// ignore unsupported charset
-	fprintf(stderr, "FICDecoder: The %s charset %d is not supported; ignoring!\n", mot ? "MOT" : "DAB", charset);
-	return "";
-}
-
-std::string FICDecoder::ConvertStringIconvToUTF8(const std::vector<uint8_t>& cleaned_data, std::string* charset_name, const std::string& src_charset) {
-	// prepare
-	iconv_t conv = iconv_open("UTF-8", src_charset.c_str());
-	if(conv == (iconv_t) -1) {
-		perror("FICDecoder: error while iconv_open");
-		return "";
-	}
-
-	size_t input_len = cleaned_data.size();
-	char input_bytes[input_len];
-	char* input = input_bytes;
-	memcpy(input_bytes, &cleaned_data[0], cleaned_data.size());
-
-	size_t output_len = input_len * 4; // theoretical worst case
-	size_t output_len_orig = output_len;
-	char output_bytes[output_len];
-	char* output = output_bytes;
-
-	// convert
-	size_t count = iconv(conv, &input, &input_len, &output, &output_len);
-	if(count == (size_t) -1) {
-		perror("FICDecoder: error while iconv");
-		return "";
-	}
-	if(input_len) {
-		fprintf(stderr, "FICDecoder: Could not convert all chars to %s!\n", src_charset.c_str());
-		return "";
-	}
-
-	if(iconv_close(conv)) {
-		perror("FICDecoder: error while iconv_close");
-		return "";
-	}
-
-	if(charset_name)
-		*charset_name = src_charset;
-	return std::string(output_bytes, output_len_orig - output_len);
-}
-
-const char* FICDecoder::no_char = "";
-const char* FICDecoder::ebu_values_0x00_to_0x1F[] = {
-		no_char , "\u0118", "\u012E", "\u0172", "\u0102", "\u0116", "\u010E", "\u0218", "\u021A", "\u010A", no_char , no_char , "\u0120", "\u0139" , "\u017B", "\u0143",
-		"\u0105", "\u0119", "\u012F", "\u0173", "\u0103", "\u0117", "\u010F", "\u0219", "\u021B", "\u010B", "\u0147", "\u011A", "\u0121", "\u013A", "\u017C", no_char
-};
-const char* FICDecoder::ebu_values_0x7B_to_0xFF[] = {
-		/* starting some chars earlier than 0x80 -----> */                                                            "\u00AB", "\u016F", "\u00BB", "\u013D", "\u0126",
-		"\u00E1", "\u00E0", "\u00E9", "\u00E8", "\u00ED", "\u00EC", "\u00F3", "\u00F2", "\u00FA", "\u00F9", "\u00D1", "\u00C7", "\u015E", "\u00DF", "\u00A1", "\u0178",
-		"\u00E2", "\u00E4", "\u00EA", "\u00EB", "\u00EE", "\u00EF", "\u00F4", "\u00F6", "\u00FB", "\u00FC", "\u00F1", "\u00E7", "\u015F", "\u011F", "\u0131", "\u00FF",
-		"\u0136", "\u0145", "\u00A9", "\u0122", "\u011E", "\u011B", "\u0148", "\u0151", "\u0150", "\u20AC", "\u00A3", "\u0024", "\u0100", "\u0112", "\u012A", "\u016A",
-		"\u0137", "\u0146", "\u013B", "\u0123", "\u013C", "\u0130", "\u0144", "\u0171", "\u0170", "\u00BF", "\u013E", "\u00B0", "\u0101", "\u0113", "\u012B", "\u016B",
-		"\u00C1", "\u00C0", "\u00C9", "\u00C8", "\u00CD", "\u00CC", "\u00D3", "\u00D2", "\u00DA", "\u00D9", "\u0158", "\u010C", "\u0160", "\u017D", "\u00D0", "\u013F",
-		"\u00C2", "\u00C4", "\u00CA", "\u00CB", "\u00CE", "\u00CF", "\u00D4", "\u00D6", "\u00DB", "\u00DC", "\u0159", "\u010D", "\u0161", "\u017E", "\u0111", "\u0140",
-		"\u00C3", "\u00C5", "\u00C6", "\u0152", "\u0177", "\u00DD", "\u00D5", "\u00D8", "\u00DE", "\u014A", "\u0154", "\u0106", "\u015A", "\u0179", "\u0164", "\u00F0",
-		"\u00E3", "\u00E5", "\u00E6", "\u0153", "\u0175", "\u00FD", "\u00F5", "\u00F8", "\u00FE", "\u014B", "\u0155", "\u0107", "\u015B", "\u017A", "\u0165", "\u0127"
-};
-
 const size_t FICDecoder::uep_sizes[] = {
 		 16,  21,  24,  29,  35,  24,  29,  35,  42,  52,  29,  35,  42,  52,  32,  42,
 		 48,  58,  70,  40,  52,  58,  70,  84,  48,  58,  70,  84, 104,  58,  70,  84,
@@ -697,28 +694,26 @@ const char* FICDecoder::languages_0x7F_downto_0x45[] = {
 		"Uzbek", "Vietnamese", "Zulu"
 };
 
-std::string FICDecoder::ConvertCharEBUToUTF8(const uint8_t value) {
-	// convert via LUT
-	if(value <= 0x1F)
-		return ebu_values_0x00_to_0x1F[value];
-	if(value >= 0x7B)
-		return ebu_values_0x7B_to_0xFF[value - 0x7B];
-
-	// convert by hand (avoiding a LUT with mostly 1:1 mapping)
-	switch(value) {
-	case 0x24:
-		return "\u0142";
-	case 0x5C:
-		return "\u016E";
-	case 0x5E:
-		return "\u0141";
-	case 0x60:
-		return "\u0104";
-	}
-
-	// leave untouched
-	return std::string((char*) &value, 1);
-}
+const char* FICDecoder::ptys_rds_0x00_to_0x1D[] = {
+		"No programme type", "News", "Current Affairs", "Information",
+		"Sport", "Education", "Drama", "Culture",
+		"Science", "Varied", "Pop Music", "Rock Music",
+		"Easy Listening Music", "Light Classical", "Serious Classical", "Other Music",
+		"Weather/meteorology", "Finance/Business", "Children's programmes", "Social Affairs",
+		"Religion", "Phone In", "Travel", "Leisure",
+		"Jazz Music", "Country Music", "National Music", "Oldies Music",
+		"Folk Music", "Documentary"
+};
+const char* FICDecoder::ptys_rbds_0x00_to_0x1D[] = {
+		"No program type", "News", "Information", "Sports",
+		"Talk", "Rock", "Classic Rock", "Adult Hits",
+		"Soft Rock", "Top 40", "Country", "Oldies",
+		"Soft", "Nostalgia", "Jazz", "Classical",
+		"Rhythm and Blues", "Soft Rhythm and Blues", "Foreign Language", "Religious Music",
+		"Religious Talk", "Personality", "Public", "College",
+		"(rfu)", "(rfu)", "(rfu)", "(rfu)",
+		"(rfu)", "Weather"
+};
 
 std::string FICDecoder::ConvertLanguageToString(const int value) {
 	if(value >= 0x00 && value <= 0x2B)
@@ -729,3 +724,35 @@ std::string FICDecoder::ConvertLanguageToString(const int value) {
 		return languages_0x7F_downto_0x45[0x7F - value];
 	return "unknown (" + std::to_string(value) + ")";
 }
+
+std::string FICDecoder::ConvertInterTableIDToString(const int value) {
+	switch(value) {
+	case 0x01:
+		return "RDS PTY";
+	case 0x02:
+		return "RBDS PTY";
+	default:
+		return "unknown";
+	}
+}
+
+std::string FICDecoder::ConvertPTYToString(const int value, const int inter_table_id) {
+	switch(inter_table_id) {
+	case 0x01:
+		return value <= 0x1D ? ptys_rds_0x00_to_0x1D[value]  : "(not used)";
+	case 0x02:
+		return value <= 0x1D ? ptys_rbds_0x00_to_0x1D[value] : "(not used)";
+	default:
+		return "(unknown)";
+	}
+}
+
+std::string FICDecoder::DeriveShortLabelUTF8(const std::string& long_label, uint16_t short_label_mask) {
+	std::string short_label;
+
+	for(size_t i = 0; i < long_label.length(); i++)		// consider discarded trailing spaces
+		if(short_label_mask & (0x8000 >> i))
+			short_label += StringTools::UTF8Substr(long_label, i, 1);
+
+	return short_label;
+}
